<?php

namespace EdSDK\FileUploaderServer\servlet;

use EdSDK\FileUploaderServer\lib\file\UtilsPHP;
use EdSDK\FileUploaderServer\lib\action\req\ReqError;
use EdSDK\FileUploaderServer\lib\action\resp\Message;
use EdSDK\FileUploaderServer\lib\action\resp\RespFail;
use EdSDK\FileUploaderServer\lib\Actions;
use EdSDK\FileUploaderServer\lib\JsonCodec;
use EdSDK\FileUploaderServer\lib\Uploader;
use Exception;

/**
 * Uploader servlet (ported from Java).
 */
class UploaderServlet {

  protected $actions;

  protected $json;

  protected $uploader;

  protected $config;

  /**
   * Creates an instance.
   */
  public function init($config) {
    $this->actions = new Actions();
    $this->json = new JsonCodec($this->actions);
    $this->config = new ServletConfig($config);
    $this->uploader = new Uploader($this->config, $this->actions);
  }

  /**
   * Gets file info.
   */
  public function getFileInfo($vector) {
    $result = [];
    foreach ($vector as $key1 => $value1) {
      foreach ($value1 as $key2 => $value2) {
        $result[$key2][$key1] = $value2;
      }
    }
    return $result;
  }

  /**
   * Gets a request.
   */
  protected function getReq($post, $files) {
    $req = NULL;
    try {
      $req = $this->json->fromJson($post['data']);
      if ($this->config->isTestAllowed()) {
        if (array_key_exists("testServerConfig", $req)) {
          $this->config->setTestConfig($req->testServerConfig);
        }
        if (array_key_exists("testClearAllFiles", $req)) {
          $this->clearAllFiles();
        }
      }
    }
    catch (Exception $e) {
      error_log($e);
      return NULL;
    }

    if (array_key_exists('file', $files)) {
      $req->file = $files['file'];
      $req->fileName = $req->file['name'];
      $req->fileSize = $req->file['size'];
    }

    return $req;
  }

  /**
   * Clears all files.
   */
  protected function clearAllFiles() {
    UtilsPHP::cleanDirectory($this->config->getTmpDir());
    UtilsPHP::cleanDirectory($this->config->getBaseDir());
  }

  /**
   * Adds headers to response.
   */
  protected function addHeaders() {
    if ($this->config->getCrossDomainUrl() != NULL && strlen($this->config->getCrossDomainUrl()) > 0) {
      header("Access-Control-Allow-Origin: " . $this->config->getCrossDomainUrl());
      header("Access-Control-Allow-Methods: POST");
      header("Access-Control-Allow-Headers: accept, content-type");
      header("Access-Control-Max-Age: 1728000");
    }
  }

  /**
   * Processes OPTIONS HTTP request.
   */
  public function doOptions() {
    $this->addHeaders();
  }

  /**
   * Processes POST HTTP request.
   */
  public function doPost($post, $files) {
    $this->addHeaders();
    $resp = NULL;
    $strResp = NULL;
    try {
      $req = NULL;

      try {
        $req = $this->getReq($post, $files);
      }
      catch (Exception $e) {
        error_log($e);
      }

      if ($req === NULL) {
        $req = new ReqError(Message::createMessage(Message::MALFORMED_REQUEST));
      }

      $resp = $this->uploader->run($req);
      if ($resp === NULL) {
        throw new Exception("Null response as result");
      }

      $strResp = $this->json->toJson($resp);
    }
    catch (Exception $e) {
      error_log($e);
      $resp = new RespFail(Message::createMessage(Message::INTERNAL_ERROR));
      $strResp = $this->json->toJson($resp);
    }

    try {
      http_response_code(200);
      header('Content-Type: application/json; charset=UTF-8');
      print($strResp);
    }
    catch (Exception $e) {
      error_log($e);
    }
  }

}
