<?php

namespace Drupal\n1ed\Plugin;

use Drupal\ckeditor\CKEditorPluginBase;
use Drupal\ckeditor\CKEditorPluginConfigurableInterface;
use Drupal\ckeditor\CKEditorPluginContextualInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\Entity\Editor;

/**
 * Defines plugin.
 *
 * @CKEditorPlugin(
 *   id = "N1EDEco",
 *   label = @Translation("N1ED"),
 *   module = "n1ed"
 * )
 */
abstract class N1EDEcosystemCKEditorPlugin extends CKEditorPluginBase implements CKEditorPluginConfigurableInterface, CKEditorPluginContextualInterface {

  /**
   * Returns plugin name.
   *
   * @return string
   *   Plugin name is a name of add-on in CKEditor terms
   */
  abstract public function getPluginName();

  /**
   * Returns module name.
   *
   * @return string
   *   Module name is a name of module in Drupal terms
   */
  abstract public function getModuleName();

  /**
   * Returns buttons list.
   *
   * @return array
   *   Associative array like in @CKEditorPlugin->getButtons
   */
  abstract public function getButtonsDef();

  /**
   * Adds controls to form using this class methods.
   */
  abstract public function addControlsToForm(&$form, $editor, $config);

  /**
   * {@inheritdoc}
   */
  public function getButtons() {
    return $this->getButtonsDef();
  }

  /**
   * {@inheritdoc}
   */
  public function getFile() {
    $apiKey = \Drupal::config('n1ed.settings')->get('apikey') ?: 'N1D8DFLT';
    // Fix for: https://www.drupal.org/project/n1ed/issues/3111919
    // Do not start URL with "https:" prefix.
    return '//cloud.n1ed.com/cdn/' . $apiKey . '/latest/ckeditor/plugins/' . $this->getPluginName() . '/plugin.js';
  }

  /**
   * {@inheritdoc}
   */
  public function getDependencies(Editor $editor) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries(Editor $editor) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function isInternal() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled(Editor $editor) {
    return TRUE;
  }

  /**
   * Returnes a value of parameter or default value.
   */
  protected function getConfigParam($settings, $name, $default, $type) {
    if (isset($settings[$name]) && is_string($settings[$name])
      && strlen($settings[$name]) > 0) {
      $value = $settings[$name];
    }
    else {
      $value = $default;
    }
    if (isset($type) && $type == 'number') {
      $value = intval($value);
    }
    else {
      if (isset($type) && $type == 'boolean') {
        if ($value === 'true' || $value === 1 || $value === TRUE) {
          $value = TRUE;
        }
        else {
          if ($value === 'false' || $value === 0 || $value === FALSE) {
            $value = FALSE;
          }
        }
      }
      else {
        if (isset($type) && $type == 'json') {
          if ($value != '') {
            $value = json_decode($value);
          }
        }
      }
    }

    $drupal8_specific_defaults = [
      'urlFileManager' => '/flmngr',
      'urlFiles' => '/sites/default/files/flmngr',
      'dirUploads' => '/',
    ];
    if (isset($drupal8_specific_defaults[$name]) && $value == '') {
      $value = $drupal8_specific_defaults[$name];
    }

    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(Editor $editor) {
    $settings = [];
    if (isset($editor->getSettings()['plugins'][$this->getPluginName()])) {
      $settings = $editor->getSettings()['plugins'][$this->getPluginName()];
    }

    return $settings;
  }

  /**
   * Adds boolean value widget to the form.
   */
  protected function addBooleanToForm(&$form, $settings, $param, $default) {
    $form[$param] = [
      '#type' => 'textfield',
      '#title' => $param,
      '#title_display' => 'invisible',
      '#default_value' => $this->getConfigParam($settings, $param, $default, 'boolean') ? "true" : "false",
      '#attributes' => [
        'style' => 'display: none!important',
        'data-n1ed-eco-param-name' => $param,
        'data-n1ed-eco-param-type' => 'boolean',
        'data-n1ed-eco-param-default' => $default ? "true" : "false",
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state, Editor $editor) {
    $config = $this->getConfig($editor);

    $form['info'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => [
        'data-n1ed-eco-plugin' => $this->getPluginName(),
        'style' => 'display: inline-block;margin-top: 13px;margin-left: 10px;',
      ],
      '#value' => '<a href="#n1ed-conf">Configure add-on</a>',
    ];

    $form['#attached']['library'][] = $this->getModuleName() . '/' . $this->getModuleName();
    $form['#attached']['drupalSettings']['n1edApiKey'] = \Drupal::config('n1ed.settings')
      ->get('apikey') ?: 'N1D8DFLT';

    $this->addControlsToForm($form, $editor, $config);

    return $form;
  }

}
